#!/usr/bin/perl
###############################################################################
#    Webmin Sysstats - users-lib.pl
#
#    Webmin Sysstats Module
#    Copyright (C) 2001 by David Bouius
#    Bug reports to: dbouius@users.sourceforge.net
#    $Id: users-lib.pl,v 1.3 2001/12/07 11:54:48 gerbier Exp $
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################

sub install_module {
  local ($minfo, $defaults) = ($_[0], $_[1]);

  local (%my_config);

  if (!-r $minfo->{'dbtables'} . "users.rrd") {
    local ($cmd);

    $cmd  = $config{'rrdtool'};
    $cmd .= " create " . $minfo->{'dbtables'} . "users.rrd";
    $cmd .= " --start 0 --step 60";
    $cmd .= " DS:users:GAUGE:100:U:U";
    $cmd .= " RRA:AVERAGE:0.5:1:70";
    $cmd .= " RRA:AVERAGE:0.5:2:750";
    $cmd .= " RRA:AVERAGE:0.5:15:650";
    $cmd .= " RRA:AVERAGE:0.5:50:650";

    `$cmd`;
  }

  if (-r $minfo->{'dbtables'} . "users.rrd") {
    $my_config{'sample_rate'} = '1';

    $my_config{'users'} = $defaults->{'user_colour'};

    &write_config($minfo, \%my_config, $defaults);

    &install_script($minfo, 'users.sh');

    return 1;
  }
  return 0;
}

sub remove_module {
}

sub mod_check_runstop {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  return $my_config->{'runstop'};
}

sub mod_runstop {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  $my_config->{'runstop'} = $my_config->{'runstop'} ? '1' : '0';
  &write_config($minfo, $my_config, $defaults);
}

sub mod_check_upgrade {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);
  return 0;
}

sub mod_upgrade {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);
}

sub graph_data {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  $rrdtool = $config{'rrdtool'};
  $sizex = $config{'graph_sizex'};
  $sizey = $config{'graph_sizey'};

  $rrd_detals  = " -v 'Users'";
  $rrd_detals .= " DEF:users=" . $minfo->{'dbtables'} . "users.rrd:users:AVERAGE";
  $rrd_detals .= " COMMENT:'Users     Max      Min  Average\\n'";
  $rrd_detals .= " LINE3:users#" . $my_config->{'users'} . ":'Users'";
  $rrd_detals .= " GPRINT:users:MAX:'%6.2lf'";
  $rrd_detals .= " GPRINT:users:MIN:'%6.2lf'";
  $rrd_detals .= " GPRINT:users:AVERAGE:'%6.2lf\\n'";

  foreach $graph ('hour', 'day', 'week', 'month') {
    if ($graph eq 'hour') {
      $time = "-3600";
      $title = "'The last hour'";
    } elsif ($graph eq 'day') {
      $time = "-86400";
      $title = "'The last day'";
    } elsif ($graph eq 'week') {
      $time = "-604800";
      $title = "'The last week'";
    } elsif ($graph eq 'month') {
      $time = "-18748800";
      $title = "'The last month'";
    }

    $common  = $rrdtool . " graph " . $minfo->{'tmpgname'} . "_" . $graph . ".png";
    $common .= " --start " . $time . " --end -60";
    $common .= " -h " . $sizey . " -w " . $sizex;
    $common .= " -a PNG --title=" . $title;

    $sig = " COMMENT:'\\n' COMMENT:\"Created: `date`\"";

    system($common . $rrd_detals . $sig);
  }
}

sub mod_config {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  print "<table width='100%'><tr><td><b>";
  print  $text{'module_subtitle'};
  print "</b></td></tr></table>\n";

  print "<form action='save_config.cgi'>\n";
  print  "<input type='hidden' name='idx' value='$in{'idx'}'>\n";

  print       "<tr><td><table width='100%'>";
  print        "<tr>";
  print         "<td width='15%'>";
  print          "<b>" . $text{'word_user'} . "</b>";
  print         "</td>";

  print         "<td width='35%'>";
  &option_colourselect('users', $my_config->{'users'},
		       $defaults->{'user_colour'}
		      );
  print         "</td>";
  print        "</tr>";
  print       "</table></td><tr>";

  print      "</table>";
  print  "<input type='submit' value='" . $text{'word_save'} . "'>";
  print "</form>\n";
}

sub mod_config_save {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  $my_config->{'users'} =
    ($in{'users2_def'} ? $in{'users2_def'} : $in{'users1_def'} eq 'default'
      ?  $defaults->{'user_colour'} : $in{'users1_def'});

  &write_config($minfo, $my_config, $defaults);
}

sub write_config {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  local (%new_config, %sh_config);

  $new_config{'sample_rate'} = $my_config->{'sample_rate'};
  $sh_config{'sample_rate'} = $my_config->{'sample_rate'};
  $new_config{'runstop'} = $my_config->{'runstop'};

  $new_config{'users'} = $my_config->{'users'};

  if (!$new_config{'runstop'}) {
    $sh_config{'users'} = 1;
  }

  unlink ($minfo->{'config_file'});
  unlink ($minfo->{'config_dir'} . 'users.cfg');

  &write_file($minfo->{'config_file'}, \%new_config);
  &write_file($minfo->{'config_dir'} . 'users.cfg', \%sh_config);
}

return 1;

